Imports ADODB

' Represents and manipulates a database. 
Public Class DBConnection
    Private m_DatabaseName As String
    Private m_Connection As Connection

    ' Close the database connection if it is open.
    Public Sub Dispose()
        CloseDatabase()
    End Sub

    ' Return the name of the open database.
    Public ReadOnly Property DatabaseName() As String
        Get
            DatabaseName = m_DatabaseName
        End Get
    End Property

    ' Return the last part of the database name.
    Public ReadOnly Property DatabaseTitle() As String
        Get
            Dim pos As Integer

            pos = InStrRev(m_DatabaseName, "\")
            If pos = 0 Then
                DatabaseTitle = m_DatabaseName
            Else
                DatabaseTitle = Mid$(m_DatabaseName, pos + 1)
            End If
        End Get
    End Property

    Public Sub OpenDatabaseAccess(ByVal database_name As String)
        ' Close the database if it is open.
        CloseDatabase()

        ' Try to open this database.
        m_Connection = New Connection()
        m_Connection.ConnectionString = _
            "Provider=Microsoft.Jet.OLEDB.4.0;" & _
            "Data Source=" & database_name & ";" & _
            "Persist Security Info=False"
        m_Connection.Open()

        ' Save the new database name.
        m_DatabaseName = database_name
    End Sub

    ' Close the database if it is open.
    Public Sub CloseDatabase()
        ' Blank the database name.
        m_DatabaseName = ""

        ' Close the connection if it is open.
        If Not m_Connection Is Nothing Then
            m_Connection.Close()
            m_Connection = Nothing
        End If
    End Sub

    ' Execute a query and return the resulting forward-only,
    ' read-only Recordset.
    Public Function ExecuteQuery(ByVal query As String) As Recordset
        Dim rs As Recordset

        ' Open the Recordset.
        rs = CType(m_Connection.Execute(query, , CommandTypeEnum.adCmdText), _
            Recordset)

        ' Return the Recordset.
        Return rs
    End Function

    ' Load a ListBox with values returned by a query.
    Public Sub LoadListBoxFromQuery(ByVal lst As ListBox, ByVal query As String, Optional ByVal separator As String = ", ")
        Dim rs As Recordset
        Dim txt As String
        Dim field_num As Integer

        lst.Items.Clear()
        rs = ExecuteQuery(query)
        Do While Not rs.EOF
            ' Combine the returned field values into a string.
            txt = New String(rs.Fields(0).Value.ToString())
            For field_num = 1 To rs.Fields.Count - 1
                txt = txt & separator & _
                    rs.Fields(field_num).Value.ToString()
            Next field_num

            ' Add the string to ListBox.
            lst.Items.Add(txt.ToString())

            ' Move to the next record.
            rs.MoveNext()
        Loop

        rs.Close()
        rs = Nothing
    End Sub

    ' Load a ComboBox with values returned by a query.
    Public Sub LoadComboBoxFromQuery(ByVal cbo As ComboBox, ByVal query As String, Optional ByVal separator As String = ", ")
        Dim rs As Recordset
        Dim txt As String
        Dim field_num As Integer

        cbo.Items.Clear()
        rs = ExecuteQuery(query)
        Do While Not rs.EOF
            ' Combine the returned field values into a string.
            txt = New String(rs.Fields(0).Value.ToString())
            For field_num = 1 To rs.Fields.Count - 1
                txt = txt & separator & _
                    rs.Fields(field_num).Value.ToString()
            Next field_num

            ' Add the string to ComboBox.
            cbo.Items.Add(txt.ToString())

            ' Move to the next record.
            rs.MoveNext()
        Loop

        rs.Close()
        rs = Nothing
    End Sub

    ' Return a Recordset containing the database's table names.
    ' The Recordset item of interest is TABLE_NAME.
    Public Function GetTableNames() As Recordset
        Dim rs As Recordset

        ' Use OpenSchema and get the table names.
        rs = CType(m_Connection.OpenSchema( _
            SchemaEnum.adSchemaTables, _
            New Object() {Nothing, Nothing, Nothing, "Table"}), _
            Recordset)

        Return rs
    End Function

    ' Return a Recordset containing the table's field names.
    ' The Recordset item of interest is COLUMN_NAME.
    Public Function GetFieldNames(ByVal table_name As String) As Recordset
        Dim rs As Recordset

        ' Use OpenSchema and get the field names.
        rs = CType(m_Connection.OpenSchema( _
            SchemaEnum.adSchemaColumns, _
            New Object() {Nothing, Nothing, table_name}), _
            Recordset)

        Return rs
    End Function

    ' Execute an action statement. Return an error message
    ' if there is a problem. Reutrn Nothing if everything is ok.
    Public Function ExecuteActionStatement(ByVal statement As String) As String
        ' Execute the statement.
        Try
            m_Connection.Execute(statement, , _
                CommandTypeEnum.adCmdText)
            Return Nothing
        Catch e As Exception
            ' Return an error message. The exception's
            ' message is not very pretty and the stack trace
            ' would overwhelm a user, but this information
            ' is ok in a prototype.
            Return e.Message & vbCrLf & vbCrLf & "Stack Trace:" & _
                vbCrLf & e.StackTrace.ToString()
        End Try
    End Function
End Class
